@@ -1,4 +1,4 @@
-# $Id: frontend.py 7339 2012-02-03 12:23:27Z milde $
+# $Id: frontend.py 8583 2020-12-01 10:53:27Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -18,8 +18,10 @@ Also exports the following functions:
 * Option callbacks: `store_multiple`, `read_config_file`.
 * Setting validators: `validate_encoding`,
   `validate_encoding_error_handler`,
-  `validate_encoding_and_error_handler`, `validate_boolean`,
-  `validate_threshold`, `validate_colon_separated_string_list`,
+  `validate_encoding_and_error_handler`,
+  `validate_boolean`, `validate_ternary`, `validate_threshold`,
+  `validate_colon_separated_list`,
+  `validate_comma_separated_list`,
   `validate_dependency_file`.
 * `make_paths_absolute`.
 * SettingSpec manipulation: `filter_settings_spec`.
@@ -31,14 +33,24 @@ import os
 import os.path
 import sys
 import warnings
-import ConfigParser as CP
 import codecs
 import optparse
 from optparse import SUPPRESS_HELP
+if sys.version_info >= (3, 0):
+    from configparser import RawConfigParser
+    from os import getcwd
+else:
+    from ConfigParser import RawConfigParser
+    from os import getcwdu as getcwd
+
 import docutils
 import docutils.utils
 import docutils.nodes
-from docutils.error_reporting import locale_encoding, ErrorOutput, ErrorString
+from docutils.utils.error_reporting import (locale_encoding, SafeString,
+                                            ErrorOutput, ErrorString)
+
+if sys.version_info >= (3, 0):
+    unicode = str  # noqa
 
 
 def store_multiple(option, opt, value, parser, *args, **kwargs):
@@ -59,7 +71,7 @@ def read_config_file(option, opt, value, parser):
     """
     try:
         new_settings = parser.get_config_file_settings(value)
-    except ValueError, error:
+    except ValueError as error:
         parser.error(error)
     parser.values.update(new_settings, parser)
 
@@ -68,9 +80,8 @@ def validate_encoding(setting, value, option_parser,
     try:
         codecs.lookup(value)
     except LookupError:
-        raise (LookupError('setting "%s": unknown encoding: "%s"'
-                           % (setting, value)),
-               None, sys.exc_info()[2])
+        raise LookupError('setting "%s": unknown encoding: "%s"'
+                          % (setting, value))
     return value
 
 def validate_encoding_error_handler(setting, value, option_parser,
@@ -78,12 +89,11 @@ def validate_encoding_error_handler(setting, value, option_parser,
     try:
         codecs.lookup_error(value)
     except LookupError:
-        raise (LookupError(
+        raise LookupError(
             'unknown encoding error handler: "%s" (choices: '
             '"strict", "ignore", "replace", "backslashreplace", '
             '"xmlcharrefreplace", and possibly others; see documentation for '
-            'the Python ``codecs`` module)' % value),
-               None, sys.exc_info()[2])
+            'the Python ``codecs`` module)' % value)
     return value
 
 def validate_encoding_and_error_handler(
@@ -110,13 +120,30 @@ def validate_encoding_and_error_handler(
 
 def validate_boolean(setting, value, option_parser,
                      config_parser=None, config_section=None):
-    if isinstance(value, unicode):
-        try:
-            return option_parser.booleans[value.strip().lower()]
-        except KeyError:
-            raise (LookupError('unknown boolean value: "%s"' % value),
-                   None, sys.exc_info()[2])
-    return value
+    """Check/normalize boolean settings:
+         True:  '1', 'on', 'yes', 'true'
+         False: '0', 'off', 'no','false', ''
+    """
+    if isinstance(value, bool):
+        return value
+    try:
+        return option_parser.booleans[value.strip().lower()]
+    except KeyError:
+        raise LookupError('unknown boolean value: "%s"' % value)
+
+def validate_ternary(setting, value, option_parser,
+                     config_parser=None, config_section=None):
+    """Check/normalize three-value settings:
+         True:  '1', 'on', 'yes', 'true'
+         False: '0', 'off', 'no','false', ''
+         any other value: returned as-is.
+    """
+    if isinstance(value, bool) or value is None:
+        return value
+    try:
+        return option_parser.booleans[value.strip().lower()]
+    except KeyError:
+        return value
 
 def validate_nonnegative_int(setting, value, option_parser,
                              config_parser=None, config_section=None):
@@ -133,18 +160,32 @@ def validate_threshold(setting, value, option_parser,
         try:
             return option_parser.thresholds[value.lower()]
         except (KeyError, AttributeError):
-            raise (LookupError('unknown threshold: %r.' % value),
-                   None, sys.exc_info[2])
+            raise LookupError('unknown threshold: %r.' % value)
 
 def validate_colon_separated_string_list(
     setting, value, option_parser, config_parser=None, config_section=None):
-    if isinstance(value, unicode):
+    if not isinstance(value, list):
         value = value.split(':')
     else:
         last = value.pop()
         value.extend(last.split(':'))
     return value
 
+def validate_comma_separated_list(setting, value, option_parser,
+                                    config_parser=None, config_section=None):
+    """Check/normalize list arguments (split at "," and strip whitespace).
+    """
+    # `value` may be ``unicode``, ``str``, or a ``list`` (when  given as
+    # command line option and "action" is "append").
+    if not isinstance(value, list):
+        value = [value]
+    # this function is called for every option added to `value`
+    # -> split the last item and append the result:
+    last = value.pop()
+    items = [i.strip(u' \t\n') for i in last.split(u',') if i.strip(u' \t\n')]
+    value.extend(items)
+    return value
+
 def validate_url_trailing_slash(
     setting, value, option_parser, config_parser=None, config_section=None):
     if not value:
@@ -163,19 +204,52 @@ def validate_dependency_file(setting, value, option_parser,
 
 def validate_strip_class(setting, value, option_parser,
                          config_parser=None, config_section=None):
-    # convert to list:
-    if isinstance(value, unicode):
-        value = [value]
-    class_values = filter(None, [v.strip() for v in value.pop().split(',')])
-    # validate:
-    for class_value in class_values:
-        normalized = docutils.nodes.make_id(class_value)
-        if class_value != normalized:
-            raise ValueError('invalid class value %r (perhaps %r?)'
-                             % (class_value, normalized))
-    value.extend(class_values)
+    # value is a comma separated string list:
+    value = validate_comma_separated_list(setting, value, option_parser,
+                                          config_parser, config_section)
+    # validate list elements:
+    for cls in value:
+        normalized = docutils.nodes.make_id(cls)
+        if cls != normalized:
+            raise ValueError('Invalid class value %r (perhaps %r?)'
+                             % (cls, normalized))
     return value
 
+def validate_smartquotes_locales(setting, value, option_parser,
+                         config_parser=None, config_section=None):
+    """Check/normalize a comma separated list of smart quote definitions.
+
+    Return a list of (language-tag, quotes) string tuples."""
+
+    # value is a comma separated string list:
+    value = validate_comma_separated_list(setting, value, option_parser,
+                                          config_parser, config_section)
+    # validate list elements
+    lc_quotes = []
+    for item in value:
+        try:
+            lang, quotes = item.split(':', 1)
+        except AttributeError:
+            # this function is called for every option added to `value`
+            # -> ignore if already a tuple:
+            lc_quotes.append(item)
+            continue
+        except ValueError:
+            raise ValueError(u'Invalid value "%s".'
+                             ' Format is "<language>:<quotes>".'
+                             % item.encode('ascii', 'backslashreplace'))
+        # parse colon separated string list:
+        quotes = quotes.strip()
+        multichar_quotes = quotes.split(':')
+        if len(multichar_quotes) == 4:
+            quotes = multichar_quotes
+        elif len(quotes) != 4:
+            raise ValueError('Invalid value "%s". Please specify 4 quotes\n'
+                '    (primary open/close; secondary open/close).'
+                             % item.encode('ascii', 'backslashreplace'))
+        lc_quotes.append((lang, quotes))
+    return lc_quotes
+
 def make_paths_absolute(pathdict, keys, base_path=None):
     """
     Interpret filesystem path settings relative to the `base_path` given.
@@ -184,7 +258,7 @@ def make_paths_absolute(pathdict, keys, base_path=None):
     `OptionParser.relative_path_settings`.
     """
     if base_path is None:
-        base_path = os.getcwdu() # type(base_path) == unicode
+        base_path = getcwd() # type(base_path) == unicode
         # to allow combining non-ASCII cwd with unicode values in `pathdict`
     for key in keys:
         if key in pathdict:
@@ -277,11 +351,10 @@ class Option(optparse.Option):
                 value = getattr(values, setting)
                 try:
                     new_value = self.validator(setting, value, parser)
-                except Exception, error:
-                    raise (optparse.OptionValueError(
+                except Exception as error:
+                    raise optparse.OptionValueError(
                         'Error in option "%s":\n    %s'
-                        % (opt, ErrorString(error))),
-                           None, sys.exc_info()[2])
+                        % (opt, ErrorString(error)))
                 setattr(values, setting, new_value)
             if self.overrides:
                 setattr(values, self.overrides, None)
@@ -313,8 +386,8 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
     thresholds = {'info': 1, 'warning': 2, 'error': 3, 'severe': 4, 'none': 5}
     """Lookup table for --report and --halt threshold values."""
 
-    booleans={'1': 1, 'on': 1, 'yes': 1, 'true': 1,
-              '0': 0, 'off': 0, 'no': 0, 'false': 0, '': 0}
+    booleans={'1': True, 'on': True, 'yes': True, 'true': True,
+              '0': False, 'off': False, 'no': False, 'false': False, '': False}
     """Lookup table for boolean configuration file settings."""
 
     default_error_encoding = getattr(sys.stderr, 'encoding',
@@ -501,8 +574,10 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
 
     config_section = 'general'
 
-    version_template = ('%%prog (Docutils %s [%s], Python %s, on %s)'
-                        % (docutils.__version__, docutils.__version_details__,
+    version_template = ('%%prog (Docutils %s%s, Python %s, on %s)'
+                        % (docutils.__version__,
+                           docutils.__version_details__ and
+                           ' [%s]'%docutils.__version_details__ or '',
                            sys.version.split()[0], sys.platform))
     """Default version message."""
 
@@ -534,8 +609,8 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         if read_config_files and not self.defaults['_disable_config']:
             try:
                 config_settings = self.get_standard_config_settings()
-            except ValueError, error:
-                self.error(error)
+            except ValueError as error:
+                self.error(SafeString(error))
             self.set_defaults_from_dict(config_settings.__dict__)
 
     def populate_from_components(self, components):
@@ -666,7 +741,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         raise KeyError('No option with dest == %r.' % dest)
 
 
-class ConfigParser(CP.RawConfigParser):
+class ConfigParser(RawConfigParser):
 
     old_settings = {
         'pep_stylesheet': ('pep_html writer', 'stylesheet'),
@@ -688,7 +763,7 @@ Skipping "%s" configuration file.
 """
 
     def __init__(self, *args, **kwargs):
-        CP.RawConfigParser.__init__(self, *args, **kwargs)
+        RawConfigParser.__init__(self, *args, **kwargs)
 
         self._files = []
         """List of paths of configuration files read."""
@@ -706,10 +781,10 @@ Skipping "%s" configuration file.
             except IOError:
                 continue
             try:
-                if sys.version_info < (3,2):
-                    CP.RawConfigParser.readfp(self, fp, filename)
+                if sys.version_info < (3, 0):
+                    RawConfigParser.readfp(self, fp, filename)
                 else:
-                    CP.RawConfigParser.read_file(self, fp, filename)
+                    RawConfigParser.read_file(self, fp, filename)
             except UnicodeDecodeError:
                 self._stderr.write(self.not_utf8_error % (filename, filename))
                 fp.close()
@@ -755,13 +830,13 @@ Skipping "%s" configuration file.
                         new_value = option.validator(
                             setting, value, option_parser,
                             config_parser=self, config_section=section)
-                    except Exception, error:
-                        raise (ValueError(
+                    except Exception as error:
+                        raise ValueError(
                             'Error in config file "%s", section "[%s]":\n'
                             '    %s\n'
                             '        %s = %s'
                             % (filename, section, ErrorString(error),
-                               setting, value)), None, sys.exc_info()[2])
+                               setting, value))
                     self.set(section, setting, new_value)
                 if option.overrides:
                     self.set(section, option.overrides, None)
