@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# $Id: manpage.py 7321 2012-01-20 06:50:44Z grubert $
+# $Id: manpage.py 8603 2021-01-08 15:24:32Z milde $
 # Author: Engelbert Gruber <grubert@users.sourceforge.net>
 # Copyright: This module is put into the public domain.
 
@@ -37,7 +37,7 @@ and
 
   AUTHOR .
 
-A unix-like system keeps an index of the DESCRIPTIONs, which is accesable
+A unix-like system keeps an index of the DESCRIPTIONs, which is accessible
 by the command whatis or apropos.
 
 """
@@ -45,6 +45,10 @@ by the command whatis or apropos.
 __docformat__ = 'reStructuredText'
 
 import re
+import sys
+
+if sys.version_info < (3, 0):
+    range = xrange  # NOQA: F821  # flake8 do not check undefined name
 
 import docutils
 from docutils import nodes, writers, languages
@@ -57,6 +61,7 @@ FIELD_LIST_INDENT = 7
 DEFINITION_LIST_INDENT = 7
 OPTION_LIST_INDENT = 7
 BLOCKQOUTE_INDENT = 3.5
+LITERAL_BLOCK_INDENT = 3.5
 
 # Define two macros so man/roff can calculate the
 # indent/unindent margins by itself
@@ -107,7 +112,7 @@ class Writer(writers.Writer):
         self.output = visitor.astext()
 
 
-class Table:
+class Table(object):
     def __init__(self):
         self._rows = []
         self._options = ['center']
@@ -128,9 +133,9 @@ class Table:
             self._coldefs.append('l')
     def _minimize_cell(self, cell_lines):
         """Remove leading and trailing blank and ``.sp`` lines"""
-        while (cell_lines and cell_lines[0] in ('\n', '.sp\n')):
+        while cell_lines and cell_lines[0] in ('\n', '.sp\n'):
             del cell_lines[0]
-        while (cell_lines and cell_lines[-1] in ('\n', '.sp\n')):
+        while cell_lines and cell_lines[-1] in ('\n', '.sp\n'):
             del cell_lines[-1]
     def as_list(self):
         text = ['.TS\n']
@@ -180,17 +185,18 @@ class Translator(nodes.NodeVisitor):
         # writing the header .TH and .SH NAME is postboned after
         # docinfo.
         self._docinfo = {
-                "title" : "", "title_upper": "",
-                "subtitle" : "",
-                "manual_section" : "", "manual_group" : "",
-                "author" : [],
-                "date" : "",
-                "copyright" : "",
-                "version" : "",
+                "title": "", "title_upper": "",
+                "subtitle": "",
+                "manual_section": "", "manual_group": "",
+                "author": [],
+                "date": "",
+                "copyright": "",
+                "version": "",
                     }
         self._docinfo_keys = []     # a list to keep the sequence as in source.
         self._docinfo_names = {}    # to get name from text not normalized.
         self._in_docinfo = None
+        self._field_name = None
         self._active_table = None
         self._in_literal = False
         self.header_written = 0
@@ -205,27 +211,28 @@ class Translator(nodes.NodeVisitor):
         #
         # Fonts are put on a stack, the top one is used.
         # ``.ft P`` or ``\\fP`` pop from stack.
+        # But ``.BI`` seams to fill stack with BIBIBIBIB...
         # ``B`` bold, ``I`` italic, ``R`` roman should be available.
         # Hopefully ``C`` courier too.
         self.defs = {
-                'indent' : ('.INDENT %.1f\n', '.UNINDENT\n'),
-                'definition_list_item' : ('.TP', ''),
-                'field_name' : ('.TP\n.B ', '\n'),
-                'literal' : ('\\fB', '\\fP'),
-                'literal_block' : ('.sp\n.nf\n.ft C\n', '\n.ft P\n.fi\n'),
+                'indent': ('.INDENT %.1f\n', '.UNINDENT\n'),
+                'definition_list_item': ('.TP', ''),
+                'field_name': ('.TP\n.B ', '\n'),
+                'literal': ('\\fB', '\\fP'),
+                'literal_block': ('.sp\n.nf\n.ft C\n', '\n.ft P\n.fi\n'),
 
-                'option_list_item' : ('.TP\n', ''),
+                'option_list_item': ('.TP\n', ''),
 
-                'reference' : (r'\fI\%', r'\fP'),
+                'reference': (r'\fI\%', r'\fP'),
                 'emphasis': ('\\fI', '\\fP'),
-                'strong' : ('\\fB', '\\fP'),
-                'term' : ('\n.B ', '\n'),
-                'title_reference' : ('\\fI', '\\fP'),
+                'strong': ('\\fB', '\\fP'),
+                'term': ('\n.B ', '\n'),
+                'title_reference': ('\\fI', '\\fP'),
 
-                'topic-title' : ('.SS ',),
-                'sidebar-title' : ('.SS ',),
+                'topic-title': ('.SS ',),
+                'sidebar-title': ('.SS ',),
 
-                'problematic' : ('\n.nf\n', '\n.fi\n'),
+                'problematic': ('\n.nf\n', '\n.fi\n'),
                     }
         # NOTE do not specify the newline before a dot-command, but ensure
         # it is there.
@@ -245,19 +252,19 @@ class Translator(nodes.NodeVisitor):
 
     def ensure_eol(self):
         """Ensure the last line in body is terminated by new line."""
-        if self.body[-1][-1] != '\n':
+        if len(self.body) > 0 and self.body[-1][-1] != '\n':
             self.body.append('\n')
 
     def astext(self):
         """Return the final formatted document as a string."""
         if not self.header_written:
             # ensure we get a ".TH" as viewers require it.
-            self.head.append(self.header())
+            self.append_header()
         # filter body
-        for i in xrange(len(self.body)-1, 0, -1):
+        for i in range(len(self.body)-1, 0, -1):
             # remove superfluous vertical gaps.
             if self.body[i] == '.sp\n':
-                if self.body[i - 1][:4] in ('.BI ','.IP '):
+                if self.body[i - 1][:4] in ('.BI ', '.IP '):
                     self.body[i] = '.\n'
                 elif (self.body[i - 1][:3] == '.B ' and
                     self.body[i - 2][:4] == '.TP\n'):
@@ -277,21 +284,21 @@ class Translator(nodes.NodeVisitor):
 
     def visit_Text(self, node):
         text = node.astext()
-        text = text.replace('\\','\\e')
+        text = text.replace('\\', '\\e')
         replace_pairs = [
-            (u'-', ur'\-'),
-            (u'\'', ur'\(aq'),
-            (u'´', ur'\''),
-            (u'`', ur'\(ga'),
+            (u'-', u'\\-'),
+            (u'\'', u'\\(aq'),
+            (u'´', u"\\'"),
+            (u'`', u'\\(ga'),
             ]
         for (in_char, out_markup) in replace_pairs:
             text = text.replace(in_char, out_markup)
         # unicode
         text = self.deunicode(text)
+        # prevent interpretation of "." at line start
+        if text.startswith('.'):
+            text = '\\&' + text
         if self._in_literal:
-            # prevent interpretation of "." at line start
-            if text[0] == '.':
-                text = '\\&' + text
             text = text.replace('\n.', '\n\\&.')
         self.body.append(text)
 
@@ -299,21 +306,21 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def list_start(self, node):
-        class enum_char:
+        class EnumChar(object):
             enum_style = {
-                    'bullet'     : '\\(bu',
-                    'emdash'     : '\\(em',
+                    'bullet': '\\(bu',
+                    'emdash': '\\(em',
                      }
 
             def __init__(self, style):
                 self._style = style
-                if node.has_key('start'):
+                if 'start' in node:
                     self._cnt = node['start'] - 1
                 else:
                     self._cnt = 0
                 self._indent = 2
                 if style == 'arabic':
-                    # indentation depends on number of childrens
+                    # indentation depends on number of children
                     # and start value.
                     self._indent = len(str(len(node.children)))
                     self._indent += len(str(self._cnt)) + 1
@@ -326,7 +333,7 @@ class Translator(nodes.NodeVisitor):
                 elif style.endswith('roman'):
                     self._indent = 5
 
-            def next(self):
+            def __next__(self):
                 if self._style == 'bullet':
                     return self.enum_style[self._style]
                 elif self._style == 'emdash':
@@ -344,15 +351,19 @@ class Translator(nodes.NodeVisitor):
                     return res.lower()
                 else:
                     return "%d." % self._cnt
+
+            if sys.version_info < (3, 0):
+                next = __next__
+
             def get_width(self):
                 return self._indent
             def __repr__(self):
                 return 'enum_style-%s' % list(self._style)
 
-        if node.has_key('enumtype'):
-            self._list_char.append(enum_char(node['enumtype']))
+        if 'enumtype' in node:
+            self._list_char.append(EnumChar(node['enumtype']))
         else:
-            self._list_char.append(enum_char('bullet'))
+            self._list_char.append(EnumChar('bullet'))
         if len(self._list_char) > 1:
             # indent nested lists
             self.indent(self._list_char[-2].get_width())
@@ -364,10 +375,10 @@ class Translator(nodes.NodeVisitor):
         self._list_char.pop()
 
     def header(self):
-        tmpl = (".TH %(title_upper)s %(manual_section)s"
+        tmpl = (".TH \"%(title_upper)s\" %(manual_section)s"
                 " \"%(date)s\" \"%(version)s\" \"%(manual_group)s\"\n"
                 ".SH NAME\n"
-                "%(title)s \- %(subtitle)s\n")
+                "%(title)s \\- %(subtitle)s\n")
         return tmpl % self._docinfo
 
     def append_header(self):
@@ -376,8 +387,8 @@ class Translator(nodes.NodeVisitor):
         # .TH title_upper section date source manual
         if self.header_written:
             return
-        self.body.append(self.header())
-        self.body.append(MACRO_DEF)
+        self.head.append(self.header())
+        self.head.append(MACRO_DEF)
         self.header_written = 1
 
     def visit_address(self, node):
@@ -387,12 +398,27 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def visit_admonition(self, node, name=None):
+        #
+        # Make admonitions a simple block quote
+        # with a strong heading
+        #
+        # Using .IP/.RE doesn't preserve indentation
+        # when admonitions contain bullets, literal,
+        # and/or block quotes.
+        #
         if name:
-            self.body.append('.IP %s\n' %
-                        self.language.labels.get(name, name))
+            # .. admonition:: has no name
+            self.body.append('.sp\n')
+            name = '%s%s:%s\n' % (
+                self.defs['strong'][0],
+                self.language.labels.get(name, name).upper(),
+                self.defs['strong'][1],
+                )
+            self.body.append(name)
+        self.visit_block_quote(node)
 
     def depart_admonition(self, node):
-        self.body.append('.RE\n')
+        self.depart_block_quote(node)
 
     def visit_attention(self, node):
         self.visit_admonition(node, 'attention')
@@ -423,7 +449,7 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def visit_block_quote(self, node):
-        # BUG/HACK: indent alway uses the _last_ indention,
+        # BUG/HACK: indent always uses the _last_ indention,
         # thus we need two of them.
         self.indent(BLOCKQOUTE_INDENT)
         self.indent(0)
@@ -450,7 +476,7 @@ class Translator(nodes.NodeVisitor):
     depart_caution = depart_admonition
 
     def visit_citation(self, node):
-        num, text = node.astext().split(None, 1)
+        num = node.astext().split(None, 1)[0]
         num = num.strip()
         self.body.append('.IP [%s] 5\n' % num)
 
@@ -557,8 +583,8 @@ class Translator(nodes.NodeVisitor):
 
     def visit_document(self, node):
         # no blank line between comment and header.
-        self.body.append(self.comment(self.document_start).rstrip()+'\n')
-        # writing header is postboned
+        self.head.append(self.comment(self.document_start).rstrip()+'\n')
+        # writing header is postponed
         self.header_written = 0
 
     def depart_document(self, node):
@@ -630,7 +656,7 @@ class Translator(nodes.NodeVisitor):
 
     def visit_field_body(self, node):
         if self._in_docinfo:
-            name_normalized = self._field_name.lower().replace(" ","_")
+            name_normalized = self._field_name.lower().replace(" ", "_")
             self._docinfo_names[name_normalized] = self._field_name
             self.visit_docinfo_item(node, name_normalized)
             raise nodes.SkipNode
@@ -695,7 +721,7 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def visit_header(self, node):
-        raise NotImplementedError, node.astext()
+        raise NotImplementedError(node.astext())
 
     def depart_header(self, node):
         pass
@@ -739,6 +765,12 @@ class Translator(nodes.NodeVisitor):
 
     depart_important = depart_admonition
 
+    def visit_inline(self, node):
+        pass
+
+    def depart_inline(self, node):
+        pass
+
     def visit_label(self, node):
         # footnote and citation
         if (isinstance(node.parent, nodes.footnote)
@@ -786,7 +818,7 @@ class Translator(nodes.NodeVisitor):
     def visit_list_item(self, node):
         # man 7 man argues to use ".IP" instead of ".TP"
         self.body.append('.IP %s %d\n' % (
-                self._list_char[-1].next(),
+                next(self._list_char[-1]),
                 self._list_char[-1].get_width(),))
 
     def depart_list_item(self, node):
@@ -799,12 +831,18 @@ class Translator(nodes.NodeVisitor):
         self.body.append(self.defs['literal'][1])
 
     def visit_literal_block(self, node):
+        # BUG/HACK: indent always uses the _last_ indention,
+        # thus we need two of them.
+        self.indent(LITERAL_BLOCK_INDENT)
+        self.indent(0)
         self.body.append(self.defs['literal_block'][0])
         self._in_literal = True
 
     def depart_literal_block(self, node):
         self._in_literal = False
         self.body.append(self.defs['literal_block'][1])
+        self.dedent()
+        self.dedent()
 
     def visit_math(self, node):
         self.document.reporter.warning('"math" role not supported',
@@ -822,11 +860,12 @@ class Translator(nodes.NodeVisitor):
     def depart_math_block(self, node):
         self.depart_literal_block(node)
 
-    def visit_meta(self, node):
-        raise NotImplementedError, node.astext()
+    # <meta> shall become an optional standard node:
+    # def visit_meta(self, node):
+    #     raise NotImplementedError(node.astext())
 
-    def depart_meta(self, node):
-        pass
+    # def depart_meta(self, node):
+    #     pass
 
     def visit_note(self, node):
         self.visit_admonition(node, 'note')
@@ -862,7 +901,7 @@ class Translator(nodes.NodeVisitor):
         # options with parameter bold italic, .BI, -f file
         #
         # we do not know if .B or .BI
-        self.context.append('.B')           # blind guess
+        self.context.append('.B ')          # blind guess. Add blank for spinx see docutils/bugs/380
         self.context.append(len(self.body)) # to be able to insert later
         self.context.append(0)              # option counter
 
@@ -876,7 +915,10 @@ class Translator(nodes.NodeVisitor):
     def visit_option(self, node):
         # each form of the option will be presented separately
         if self.context[-1] > 0:
-            self.body.append(', ')
+            if self.context[-3] == '.BI':
+                self.body.append('\\fR,\\fB ')
+            else:
+                self.body.append('\\fP,\\fB ')
         if self.context[-3] == '.BI':
             self.body.append('\\')
         self.body.append(' ')
@@ -925,13 +967,16 @@ class Translator(nodes.NodeVisitor):
         # ``.PP`` : Start standard indented paragraph.
         # ``.LP`` : Start block paragraph, all except the first.
         # ``.P [type]``  : Start paragraph type.
-        # NOTE dont use paragraph starts because they reset indentation.
+        # NOTE do not use paragraph starts because they reset indentation.
         # ``.sp`` is only vertical space
         self.ensure_eol()
         if not self.first_child(node):
             self.body.append('.sp\n')
+        # set in literal to escape dots after a new-line-character
+        self._in_literal = True
 
     def depart_paragraph(self, node):
+        self._in_literal = False
         self.body.append('\n')
 
     def visit_problematic(self, node):
@@ -1109,7 +1154,7 @@ class Translator(nodes.NodeVisitor):
         pass
 
     def depart_rubric(self, node):
-        pass
+        self.body.append('\n')
 
     def visit_transition(self, node):
         # .PP      Begin a new paragraph and reset prevailing indent.
