@@ -1,4 +1,4 @@
-# $Id: DocutilsTestSupport.py 7440 2012-06-13 14:14:12Z milde $
+# $Id: DocutilsTestSupport.py 8601 2021-01-03 21:58:06Z milde $
 # Authors: David Goodger <goodger@python.org>;
 #          Garth Kidd <garth@deadlybloodyserious.com>
 # Copyright: This module has been placed in the public domain.
@@ -9,7 +9,7 @@ Exports the following:
 :Modules:
     - `statemachine` is 'docutils.statemachine'
     - `nodes` is 'docutils.nodes'
-    - `urischemes` is 'docutils.urischemes'
+    - `urischemes` is 'docutils.utils.urischemes'
     - `utils` is 'docutils.utils'
     - `transforms` is 'docutils.transforms'
     - `states` is 'docutils.parsers.rst.states'
@@ -38,6 +38,7 @@ Exports the following:
     - `HtmlFragmentTestSuite`
     - `DevNull` (output sink)
 """
+from __future__ import print_function
 __docformat__ = 'reStructuredText'
 
 import sys
@@ -46,18 +47,13 @@ import unittest
 import re
 import inspect
 import traceback
+import warnings
 from pprint import pformat
 
 testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
 os.chdir(testroot)
-if sys.version_info >= (3,0):
-    sys.path.insert(0, os.path.normpath(os.path.join(testroot,
-                                                     '..', 'build', 'lib')))
-    sys.path.append(os.path.normpath(os.path.join(testroot, '..',
-                                                  'build', 'lib', 'extras')))
-else:
-    sys.path.insert(0, os.path.normpath(os.path.join(testroot, '..')))
-    sys.path.append(os.path.normpath(os.path.join(testroot, '..', 'extras')))
+sys.path.insert(0, os.path.normpath(os.path.join(testroot, '..')))
+sys.path.append(os.path.normpath(os.path.join(testroot, '..', 'extras')))
 sys.path.insert(0, testroot)
 
 try:
@@ -65,13 +61,13 @@ try:
     import package_unittest
     import docutils
     import docutils.core
-    from docutils import frontend, nodes, statemachine, urischemes, utils
+    from docutils import frontend, nodes, statemachine, utils
+    from docutils.utils import urischemes
     from docutils.transforms import universal
-    from docutils.parsers import rst
+    from docutils.parsers import rst, recommonmark_wrapper
     from docutils.parsers.rst import states, tableparser, roles, languages
     from docutils.readers import standalone, pep
     from docutils.statemachine import StringList, string2lines
-    from docutils._compat import bytes
 except ImportError:
     # The importing module (usually __init__.py in one of the
     # subdirectories) may catch ImportErrors in order to detect the
@@ -87,12 +83,15 @@ try:
 except:
     import pdb
 
+if sys.version_info >= (3, 0):
+    unicode = str  # noqa
+
 
 # Hack to make repr(StringList) look like repr(list):
 StringList.__repr__ = StringList.__str__
 
 
-class DevNull:
+class DevNull(object):
 
     """Output sink."""
 
@@ -118,20 +117,16 @@ class StandardTestCase(unittest.TestCase):
            operator.
         """
         if not first == second:
-            raise self.failureException, \
-                  (msg or '%s != %s' % _format_str(first, second))
+            raise self.failureException(
+                msg or '%s != %s' % _format_str(first, second))
 
     def assertNotEqual(self, first, second, msg=None):
         """Fail if the two objects are equal as determined by the '=='
            operator.
         """
         if first == second:
-            raise self.failureException, \
-                  (msg or '%s == %s' % _format_str(first, second))
-
-    # python 2.3
-    if not hasattr(unittest.TestCase, "assertTrue"):
-        assertTrue = unittest.TestCase.failUnless
+            raise self.failureException(
+                msg or '%s == %s' % _format_str(first, second))
 
     # aliases for assertion methods, deprecated since Python 2.7
 
@@ -201,7 +196,7 @@ class CustomTestCase(StandardTestCase):
         """`input`, `output`, and `expected` should all be strings."""
         if isinstance(input, unicode):
             input = input.encode('raw_unicode_escape')
-        if sys.version_info > (3,):
+        if sys.version_info > (3, 0):
             # API difference: Python 3's node.__str__ doesn't escape
             #assert expected is None or isinstance(expected, unicode)
             if isinstance(expected, bytes):
@@ -220,18 +215,18 @@ class CustomTestCase(StandardTestCase):
             output = '\n'.join(output.splitlines())
         try:
             self.assertEqual(output, expected)
-        except AssertionError, error:
-            print >>sys.stderr, '\n%s\ninput:' % (self,)
-            print >>sys.stderr, input
+        except AssertionError as error:
+            print('\n%s\ninput:' % (self,), file=sys.stderr)
+            print(input, file=sys.stderr)
             try:
                 comparison = ''.join(self.compare(expected.splitlines(1),
                                                   output.splitlines(1)))
-                print >>sys.stderr, '-: expected\n+: output'
-                print >>sys.stderr, comparison
+                print('-: expected\n+: output', file=sys.stderr)
+                print(comparison, file=sys.stderr)
             except AttributeError:      # expected or output not a string
                 # alternative output for non-strings:
-                print >>sys.stderr, 'expected: %r' % expected
-                print >>sys.stderr, 'output:   %r' % output
+                print('expected: %r' % expected, file=sys.stderr)
+                print('output:   %r' % output, file=sys.stderr)
             raise error
 
 
@@ -366,20 +361,20 @@ class TransformTestCase(CustomTestCase):
     def test_transforms_verbosely(self):
         if self.run_in_debugger:
             pdb.set_trace()
-        print '\n', self.id
-        print '-' * 70
-        print self.input
+        print('\n', self.id)
+        print('-' * 70)
+        print(self.input)
         settings = self.settings.copy()
         settings.__dict__.update(self.suite_settings)
         document = utils.new_document('test data', settings)
         self.parser.parse(self.input, document)
-        print '-' * 70
-        print document.pformat()
+        print('-' * 70)
+        print(document.pformat())
         for transformClass in self.transforms:
             transformClass(document).apply()
         output = document.pformat()
-        print '-' * 70
-        print output
+        print('-' * 70)
+        print(output)
         self.compare_output(self.input, output, self.expected)
 
 
@@ -523,6 +518,34 @@ class PEPParserTestSuite(ParserTestSuite):
     test_case_class = PEPParserTestCase
 
 
+class RecommonmarkParserTestCase(ParserTestCase):
+
+    """Recommonmark-specific parser test case."""
+
+    parser = recommonmark_wrapper.Parser()
+    """Parser shared by all RecommonmarkParserTestCases."""
+
+    option_parser = frontend.OptionParser(
+                        components=(recommonmark_wrapper.Parser,))
+    settings = option_parser.get_default_values()
+    settings.report_level = 5
+    settings.halt_level = 5
+    settings.debug = package_unittest.debug
+
+class RecommonmarkParserTestSuite(ParserTestSuite):
+
+    """A collection of RecommonmarkParserTestCases."""
+
+    test_case_class = RecommonmarkParserTestCase
+
+    def generateTests(self, dict, dictname='totest'):
+        if 'recommonmark' not in recommonmark_wrapper.Parser.supported:
+            return
+        # suppress UserWarnings from recommonmark parser
+        warnings.filterwarnings('ignore', message='Unsupported.*type')
+        ParserTestSuite.generateTests(self, dict, dictname='totest')
+
+
 class GridTableParserTestCase(CustomTestCase):
 
     parser = tableparser.GridTableParser()
@@ -533,7 +556,7 @@ class GridTableParserTestCase(CustomTestCase):
             self.parser.find_head_body_sep()
             self.parser.parse_table()
             output = self.parser.cells
-        except Exception, details:
+        except Exception as details:
             output = '%s: %s' % (details.__class__.__name__, details)
         self.compare_output(self.input, pformat(output) + '\n',
                             pformat(self.expected) + '\n')
@@ -542,7 +565,7 @@ class GridTableParserTestCase(CustomTestCase):
         try:
             output = self.parser.parse(StringList(string2lines(self.input),
                                                   'test data'))
-        except Exception, details:
+        except Exception as details:
             output = '%s: %s' % (details.__class__.__name__, details)
         self.compare_output(self.input, pformat(output) + '\n',
                             pformat(self.expected) + '\n')
@@ -629,76 +652,14 @@ class SimpleTableParserTestSuite(CustomTestSuite):
                                  run_in_debugger=run_in_debugger)
 
 
-class PythonModuleParserTestCase(CustomTestCase):
-
-    def test_parser(self):
-        if self.run_in_debugger:
-            pdb.set_trace()
-        try:
-            import compiler
-        except ImportError:
-            # skip on Python 3
-            return
-        from docutils.readers.python import moduleparser
-        module = moduleparser.parse_module(self.input, 'test data').pformat()
-        output = str(module)
-        self.compare_output(self.input, output, self.expected)
-
-    def test_token_parser_rhs(self):
-        if self.run_in_debugger:
-            pdb.set_trace()
-        try:
-            import compiler
-        except ImportError:
-            # skip on Python 3
-            return
-        from docutils.readers.python import moduleparser
-        tr = moduleparser.TokenParser(self.input)
-        output = tr.rhs(1)
-        self.compare_output(self.input, output, self.expected)
-
-
-class PythonModuleParserTestSuite(CustomTestSuite):
-
-    """
-    A collection of PythonModuleParserTestCase.
-    """
-
-    def generateTests(self, dict, dictname='totest',
-                      testmethod='test_parser'):
-        """
-        Stock the suite with test cases generated from a test data dictionary.
-
-        Each dictionary key (test type's name) maps to a list of tests. Each
-        test is a list: input, expected output, optional modifier. The
-        optional third entry, a behavior modifier, can be 0 (temporarily
-        disable this test) or 1 (run this test under the pdb debugger). Tests
-        should be self-documenting and not require external comments.
-        """
-        for name, cases in dict.items():
-            for casenum in range(len(cases)):
-                case = cases[casenum]
-                run_in_debugger = False
-                if len(case)==3:
-                    if case[2]:
-                        run_in_debugger = True
-                    else:
-                        continue
-                self.addTestCase(
-                      PythonModuleParserTestCase, testmethod,
-                      input=case[0], expected=case[1],
-                      id='%s[%r][%s]' % (dictname, name, casenum),
-                      run_in_debugger=run_in_debugger)
-
-
 class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
 
     """
     Test case for publish.
     """
 
-    settings_default_overrides = {'_disable_config': 1,
-                                  'strict_visitor': 1}
+    settings_default_overrides = {'_disable_config': True,
+                                  'strict_visitor': True}
     writer_name = '' # set in subclasses or constructor
 
     def __init__(self, *args, **kwargs):
@@ -749,28 +710,6 @@ class PublishTestSuite(CustomTestSuite):
                       writer_name=self.writer_name)
 
 
-class HtmlPublishPartsTestSuite(CustomTestSuite):
-
-    def generateTests(self, dict, dictname='totest'):
-        for name, (settings_overrides, cases) in dict.items():
-            settings = self.suite_settings.copy()
-            settings.update(settings_overrides)
-            for casenum in range(len(cases)):
-                case = cases[casenum]
-                run_in_debugger = False
-                if len(case)==3:
-                    if case[2]:
-                        run_in_debugger = True
-                    else:
-                        continue
-                self.addTestCase(
-                      HtmlWriterPublishPartsTestCase, 'test_publish',
-                      input=case[0], expected=case[1],
-                      id='%s[%r][%s]' % (dictname, name, casenum),
-                      run_in_debugger=run_in_debugger,
-                      suite_settings=settings)
-
-
 class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
 
     """
@@ -831,15 +770,10 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
             self.standard_html_meta_value, '...')
         parts['html_prolog'] = parts['html_prolog'].replace(
             self.standard_html_prolog, '')
-        # remove empty values:
-        for key in parts.keys():
-            if not parts[key]:
-                del parts[key]
-        # standard output format:
-        keys = parts.keys()
-        keys.sort()
         output = []
-        for key in keys:
+        for key in sorted(parts.keys()):
+            if not parts[key]:
+                continue
             output.append("%r: '''%s'''"
                           % (key, parts[key]))
             if output[-1].endswith("\n'''"):
@@ -847,6 +781,29 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
         return '{' + ',\n '.join(output) + '}\n'
 
 
+class HtmlPublishPartsTestSuite(CustomTestSuite):
+
+    testcase_class = HtmlWriterPublishPartsTestCase
+
+    def generateTests(self, dict, dictname='totest'):
+        for name, (settings_overrides, cases) in dict.items():
+            settings = self.suite_settings.copy()
+            settings.update(settings_overrides)
+            for casenum in range(len(cases)):
+                case = cases[casenum]
+                run_in_debugger = False
+                if len(case)==3:
+                    if case[2]:
+                        run_in_debugger = True
+                    else:
+                        continue
+                self.addTestCase(self.testcase_class, 'test_publish',
+                                 input=case[0], expected=case[1],
+                                 id='%s[%r][%s]' % (dictname, name, casenum),
+                                 run_in_debugger=run_in_debugger,
+                                 suite_settings=settings)
+
+
 def exception_data(func, *args, **kwds):
     """
     Execute `func(*args, **kwds)` and return the resulting exception, the
@@ -854,7 +811,7 @@ def exception_data(func, *args, **kwds):
     """
     try:
         func(*args, **kwds)
-    except Exception, detail:
+    except Exception as detail:
         return (detail, detail.args,
                 '%s: %s' % (detail.__class__.__name__, detail))
 
