@@ -1,19 +1,27 @@
 #! /usr/bin/env python
 
-# $Id: test_include.py 8164 2017-08-14 11:28:48Z milde $
+# $Id: test_include.py 8595 2020-12-15 23:06:58Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for misc.py "include" directive.
 """
+from __future__ import absolute_import
 
 import os.path
 import sys
-from __init__ import DocutilsTestSupport
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 from docutils.parsers.rst import states
+from docutils.parsers.recommonmark_wrapper import with_recommonmark
 from docutils.utils.code_analyzer import with_pygments
 
+if sys.version_info >= (3, 0):
+    unichr = chr  # noqa
+
+
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
     if not with_pygments:
@@ -38,11 +46,14 @@ include11 = mydir('include 11.txt')
 include12 = mydir('include12.txt')
 include13 = mydir('include13.txt')
 include14 = mydir('includes/include14.txt')
+include15 = mydir('includes/include15.txt')
+include16 = mydir('includes/include16.txt')
 include_literal = mydir('include_literal.txt')
+include_md = mydir('include.md')
 utf_16_file = mydir('utf-16.csv')
 utf_16_error_str = ("UnicodeDecodeError: 'ascii' codec can't decode byte 0xfe "
                     "in position 0: ordinal not in range(128)")
-if sys.version_info < (3,0):
+if sys.version_info < (3, 0):
     utf_16_error_str = ("UnicodeError: Unable to decode input data.  "
                         "Tried the following encodings: 'ascii'.\n"
                         "            (%s)" % utf_16_error_str)
@@ -63,6 +74,32 @@ except:
 InputError: [Errno 2] No such file or directory: '\u043c\u0438\u0440.txt'.\
 """
 
+# Parsing with Markdown (recommonmark) is an optional feature depending
+# on 3rd-party modules:
+if with_recommonmark:
+    markdown_parsing_result = """\
+    <section ids="title-1" names="title\\ 1">
+        <title>
+            Title 1
+        <paragraph>
+            <emphasis>
+                emphasis
+             and \n\
+            <emphasis>
+                also emphasis
+        <paragraph>
+            No whitespace required around a
+            <reference name="phrase reference" refuri="/uri">
+                phrase reference
+            ."""
+else:
+    markdown_parsing_result = """\
+    <system_message level="2" source="test_parsers/test_rst/test_directives/include.md" type="WARNING">
+        <paragraph>
+            Missing dependency: MarkDown input is processed by a 3rd party parser but Python did not find the required module "recommonmark" (https://pypi.org/project/recommonmark/).\
+"""
+
+
 totest = {}
 
 totest['include'] = [
@@ -76,10 +113,10 @@ A paragraph.
 """ % include1,
 """\
 <document source="test data">
-    <section ids="include-test" names="include\ test">
+    <section ids="include-test" names="include\\ test">
         <title>
             Include Test
-        <section ids="inclusion-1" names="inclusion\ 1">
+        <section ids="inclusion-1" names="inclusion\\ 1">
             <title>
                 Inclusion 1
             <paragraph>
@@ -103,10 +140,10 @@ A paragraph.
 """ % include1,
 """\
 <document source="test data">
-    <section ids="include-test" names="include\ test">
+    <section ids="include-test" names="include\\ test">
         <title>
             Include Test
-        <literal_block classes="test" ids="my-name" names="my\ name" source="%s" xml:space="preserve">
+        <literal_block classes="test" ids="my-name" names="my\\ name" source="%s" xml:space="preserve">
             Inclusion 1
             -----------
             \n\
@@ -151,7 +188,7 @@ Include code
 <document source="test data">
     <paragraph>
         Include code
-    <literal_block classes="code test" ids="my-name" names="my\ name" source="%s" xml:space="preserve">
+    <literal_block classes="code test" ids="my-name" names="my\\ name" source="%s" xml:space="preserve">
         Inclusion 1
         -----------
         \n\
@@ -183,6 +220,22 @@ Include code, add line numbers
         This file is used by ``test_include.py``.
 """ % reldir(include1)],
 ["""\
+Include markdown (recommonmark).
+
+.. include:: %s
+   :parser: markdown
+
+A paragraph.
+""" % include_md,
+"""\
+<document source="test data">
+    <paragraph>
+        Include markdown (recommonmark).
+%s
+    <paragraph>
+        A paragraph.
+""" % markdown_parsing_result],
+["""\
 Let's test the parse context.
 
     This paragraph is in a block quote.
@@ -219,7 +272,7 @@ A paragraph.
 """,
 """\
 <document source="test data">
-    <section ids="include-test" names="include\ test">
+    <section ids="include-test" names="include\\ test">
         <title>
             Include Test
         <system_message level="4" line="4" source="test data" type="SEVERE">
@@ -243,10 +296,10 @@ A paragraph.
 """ % (include1, include1),
 """\
 <document source="test data">
-    <section ids="include-test" names="include\ test">
+    <section ids="include-test" names="include\\ test">
         <title>
             Include Test
-        <section dupnames="inclusion\ 1" ids="inclusion-1">
+        <section dupnames="inclusion\\ 1" ids="inclusion-1">
             <title>
                 Inclusion 1
             <paragraph>
@@ -254,7 +307,7 @@ A paragraph.
                 <literal>
                     test_include.py
                 .
-        <section dupnames="inclusion\ 1" ids="id1">
+        <section dupnames="inclusion\\ 1" ids="id1">
             <title>
                 Inclusion 1
             <system_message backrefs="id1" level="1" line="2" source="%s" type="INFO">
@@ -282,10 +335,10 @@ A paragraph.
 """ % (include1, include1),
 """\
 <document source="test data">
-    <section ids="include-test" names="include\ test">
+    <section ids="include-test" names="include\\ test">
         <title>
             Include Test
-        <section dupnames="inclusion\ 1" ids="inclusion-1">
+        <section dupnames="inclusion\\ 1" ids="inclusion-1">
             <title>
                 Inclusion 1
             <paragraph>
@@ -294,7 +347,7 @@ A paragraph.
                     test_include.py
                 .
             <transition>
-        <section dupnames="inclusion\ 1" ids="id1">
+        <section dupnames="inclusion\\ 1" ids="id1">
             <title>
                 Inclusion 1
             <system_message backrefs="id1" level="1" line="2" source="%s" type="INFO">
@@ -558,7 +611,7 @@ Testing errors in included file:
                 .. admonition::
                 
                    without title and content following a blank line
-    <section ids="section-underline-too-short" names="section\ underline\ too\ short">
+    <section ids="section-underline-too-short" names="section\\ underline\\ too\\ short">
         <title>
             section underline too short
         <system_message level="2" line="36" source="%(source)s" type="WARNING">
@@ -1017,6 +1070,59 @@ Including includes/include14.txt
             ::
          ../sibling/include7.txt
 """ % reldir(include6)],
+["""\
+Circular inclusion
+
+.. include:: %s
+""" % include15,
+"""\
+<document source="test data">
+    <paragraph>
+        Circular inclusion
+    <paragraph>
+        File "include15.txt": example of rekursive inclusion.
+    <paragraph>
+        File "include16.txt": example of rekursive inclusion.
+    <system_message level="2" line="3" source="%s" type="WARNING">
+        <paragraph>
+            circular inclusion in "include" directive: %s < %s < %s < test data
+        <literal_block xml:space="preserve">
+            .. include:: include15.txt
+    <paragraph>
+        No loop when clipping before the "include" directive:
+    <paragraph>
+        File "include15.txt": example of rekursive inclusion.
+""" % (reldir(include16), reldir(include15),
+       reldir(include16), reldir(include15))],
+["""\
+Circular inclusion with clipping.
+
+.. include:: %s
+   :start-line: 2
+""" % include16,
+"""\
+<document source="test data">
+    <paragraph>
+        Circular inclusion with clipping.
+    <paragraph>
+        File "include15.txt": example of rekursive inclusion.
+    <paragraph>
+        File "include16.txt": example of rekursive inclusion.
+    <system_message level="2" line="3" source="%s" type="WARNING">
+        <paragraph>
+            circular inclusion in "include" directive: %s < %s < %s < %s < test data
+        <literal_block xml:space="preserve">
+            .. include:: include15.txt
+    <paragraph>
+        No loop when clipping before the "include" directive:
+    <paragraph>
+        File "include15.txt": example of rekursive inclusion.
+    <paragraph>
+        No loop when clipping before the "include" directive:
+    <paragraph>
+        File "include15.txt": example of rekursive inclusion.
+""" % (reldir(include16), reldir(include15), reldir(include16),
+       reldir(include15), reldir(include16))],
 ]
 
 if __name__ == '__main__':
