@@ -1,16 +1,18 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.4 $
-# Date: $Date: 2002/10/09 00:51:53 $
+# $Id: test_option_lists.py 8481 2020-01-31 08:17:24Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for states.py.
 """
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -677,8 +679,97 @@ These next ones should be simple paragraphs:
     <paragraph>
         -1 and this one too.
 """],
+["""\
+Complex optargs:
+
+--source-url=<URL>      Use the supplied <URL> verbatim.
+--output-encoding=<name[:handler]>, -o<name[:handler]>
+                        Specify the text encoding for output.
+--af=<filter1[=parameter1:parameter2:...],filter2,...>
+                        Setup a chain of audio filters.
+                        Option argument containing delimiter ``=``.
+-f <[path]filename>     Send output to file.
+-d <src dest>           Use diff from <src> to <dest>.
+--bogus=<x y z>         Bogus 3D coordinates.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Complex optargs:
+    <option_list>
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        --source-url
+                    <option_argument delimiter="=">
+                        <URL>
+            <description>
+                <paragraph>
+                    Use the supplied <URL> verbatim.
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        --output-encoding
+                    <option_argument delimiter="=">
+                        <name[:handler]>
+                <option>
+                    <option_string>
+                        -o
+                    <option_argument delimiter="">
+                        <name[:handler]>
+            <description>
+                <paragraph>
+                    Specify the text encoding for output.
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        --af
+                    <option_argument delimiter="=">
+                        <filter1[=parameter1:parameter2:...],filter2,...>
+            <description>
+                <paragraph>
+                    Setup a chain of audio filters.
+                    Option argument containing delimiter \n\
+                    <literal>
+                        =
+                    .
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        -f
+                    <option_argument delimiter=" ">
+                        <[path]filename>
+            <description>
+                <paragraph>
+                    Send output to file.
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        -d
+                    <option_argument delimiter=" ">
+                        <src dest>
+            <description>
+                <paragraph>
+                    Use diff from <src> to <dest>.
+        <option_list_item>
+            <option_group>
+                <option>
+                    <option_string>
+                        --bogus
+                    <option_argument delimiter="=">
+                        <x y z>
+            <description>
+                <paragraph>
+                    Bogus 3D coordinates.
+"""],
 ]
 
+
 if __name__ == '__main__':
     import unittest
     unittest.main(defaultTest='suite')
